/*
   Semax Smartmeter with DMSR
   This example is based on the ideas of https://forum.arduino.cc/t/dezimalzahl-aus-char-datensatz-auslesen/934748
   https://www.netbeheernederland.nl/_upload/Files/Slimme_meter_15_a727fce1f1.pdf

   by noiasca

   2021-12-12 initial version
*/

#include <NoiascaSmartmeter.h>           // download from https://werner.rothschopf.net/

SmartmeterP1 smartmeter(Serial);         // use HW Serial if available

// The P1 protocol on the Semax/Elster AM103 is specified for 115200 Baud.
// this exceeds the recommendation for AltSoftSerial (or any other Software Serial)
//#include <AltSoftSerial.h>
//AltSoftSerial altSerial;               // AltSoftSerial always uses RX = 8; TX = 9 on Uno/Nano.
//SmartmeterP1 smartmeter(altSerial);    // receive data from Smartmeter with altSerial

// user sketch specific variables and functions
constexpr byte indicatorPin = 13;        // pin to be switched
const float threasholdCurrent = 0.05;    // if this value is exceeded a pin will be switched

// this function will be called each time we have received a new packet (= several lines)
void callbackOnNewData()
{
  printAllData();
  if (smartmeter.getActualElectricityPowerReceived() > threasholdCurrent)
  {
    Serial.println(F("true - lower"));
    digitalWrite(indicatorPin, LOW);
  }
  else
  {
    Serial.println(F("false - higher"));
    digitalWrite(indicatorPin, HIGH);
  }
}

// this function prints all available data
void printAllData()
{
  Serial.print(F("timestamp                       ")); Serial.println(smartmeter.getTimestamp());
  Serial.print(F("equipmentIdentifier             ")); Serial.println(smartmeter.getEquipmentIdentifier());
  for (byte i = 0; i < tarifs; i++)                // this device uses up to two tarifs in the P1 protocol, hence the for loop
  {
    Serial.print(F("electricityDeliveredToClient[")); Serial.print(i); Serial.print("]  "); Serial.println(smartmeter.getElectricityDeliveredToClient(i));
    Serial.print(F("electricityDeliveredByClient[")); Serial.print(i); Serial.print("]  "); Serial.println(smartmeter.getElectricityDeliveredByClient(i));
  }
  Serial.print(F("instantaneousVoltageL1           ")); Serial.println(smartmeter.getInstantaneousVoltageL1());
  Serial.print(F("instantaneousVoltageL2           ")); Serial.println(smartmeter.getInstantaneousVoltageL2());
  Serial.print(F("instantaneousVoltageL3           ")); Serial.println(smartmeter.getInstantaneousVoltageL3());
  Serial.print(F("instantaneousCurrentL1           ")); Serial.println(smartmeter.getInstantaneousCurrentL1());
  Serial.print(F("instantaneousCurrentL2           ")); Serial.println(smartmeter.getInstantaneousCurrentL2());
  Serial.print(F("instantaneousCurrentL3           ")); Serial.println(smartmeter.getInstantaneousCurrentL3());
  Serial.print(F("actualElectricityPowerReceived   ")); Serial.println(smartmeter.getActualElectricityPowerReceived());
  Serial.print(F("actualElectricityPowerDelivered  ")); Serial.println(smartmeter.getActualElectricityPowerDelivered());
  Serial.print(F("isDirty                          ")); Serial.println(smartmeter.isDirty());
}

// Arduino setup and loop
void setup() {
  Serial.begin(115200);
  Serial.println(F("Smartmeter Semex / Elster AM103"));
  pinMode(indicatorPin, OUTPUT);
  //altSerial.begin(115200);                  // start your interface (if necessary)
  smartmeter.setOnNewData(callbackOnNewData); // register a callback onNewData
}

void loop() {
  smartmeter.update();   // call the object in loop to read data
}
